import shelve
import tempfile
import os
import collections

class TempContext:
	def __init__(self):
		self.tmp_files = []
		
	def __del__(self):
		self.clear()
		
	def new_file(self, ext = ''):
		fd, tmp = tempfile.mkstemp(ext)
		os.close(fd)
		self.tmp_files.append(tmp)
		return tmp
		
	def clear(self):
		for f in self.tmp_files:
			try:
				os.remove(f)
			except OSError:
				pass

## Singleton implementation of a registry class
#
class Session:
	__shared_state = {
		'doc_uid' : 0,
		'doc_dict' : {},
		'history' : collections.deque(maxlen=20),
		'future'  : [],
		'has_changed' : False,
		'filename' : None,
		'temp' : {}
	}
	
	def __init__(self, main_win = None):
		self.__dict__ = self.__shared_state
		if main_win: self.main = main_win

	def assign_uid(self):
		self.doc_uid += 1
		return self.doc_uid - 1
	
	def can_redo(self):
		can = bool(self.future)
		self.main.ui.tb_redo.set_sensitive(can)
		return can
	
	def can_undo(self):
		can = bool(self.history)
		self.main.ui.tb_undo.set_sensitive(can)
		return can
	
	def clear(self):
		self.doc_uid = 0
		self.doc_dict = {}
		self.history = collections.deque(maxlen=20),
		self.has_changed = True

	def do(self, cmd):
		cmd.do()
		self.history.append(cmd)
		self.future = []
		
		self.can_redo()
		return self.can_undo()

	def new(self):
		self.doc_uid = 0
		self.doc_dict = {}
		self.history = collections.deque(maxlen=20),
		self.has_changed = False
		self.filename = None

	def save(self, fn = None):
		if fn:
			filename = fn
			self.has_changed = True
		else: filename = self.filename
		if not filename: return False
		if not self.has_changed:
			self.status("No changes need to be saved.")
			return False

		try:
			shelf = shelve.open(filename, 'n')
			shelf['ls_doc'] = self.ls_doc
			shelf['doc_dict'] = self.doc_dict
			shelf['doc_uid'] = self.doc_uid
			shelf['history'] = self.history
			shelf.close()
			self.filename = filename
			self.status('File "%s" has been saved successfully.' % self.filename)
		except IOError:
			self.status('Unable to save "%s".' % self.filename)
			return False
			
		self.has_changed = False
		return True
		
	def open(self, filename):
		shelf = shelve.open(filename)
		try:
			model = shelf['ts_plot']
			##TODO: Probably a check on model is needed here
			self.ls_plot = model
			self.doc_dict = shelf['doc_dict']
			self.doc_uid = shelf['doc_uid']
			self.state = shelf['state'] if 'state' in shelf else ""
			self.history = shelf['history'] if 'history' in shelf else []
			
			self.filename = filename
			self.has_changed = False
			
		except (KeyError, ValueError):
			return False

		finally:
			shelf.close()
			self.status('File "%s" has been loaded successfully.' % self.filename)

		return True

	def redo(self):
		if not self.can_redo(): return False
		
		cmd = self.future.pop()
		cmd.redo()
		self.history.append(cmd)
		
		self.can_undo()
		return self.can_redo()
	
	def temp_new(self, name):
		self.temp[name] = TempContext()
		return self.temp[name]
		
	def temp_delete(self, name):
		del self.temp[name]

	def undo(self):
		if not self.can_undo(): return False
		
		cmd = self.history.pop()
		cmd.undo()
		self.future.append(cmd)
		
		self.can_redo()
		return self.can_undo()
